@@ -37,7 +37,6 @@
 #include <kpushbutton.h>
 #include <kcombobox.h>
 
-#include "groupconfiggui.h"
 #include "groupconfigdlg.h"
 
 
@@ -48,6 +47,17 @@ static void removeList(QList<KUser> & from, const QList<KUser> & that);
 static bool userMod(const QString & user, const QList<KUserGroup> & groups);
 
 
+GroupConfigGUI::GroupConfigGUI( QWidget *parent )
+    : QWidget( parent )
+{
+    setupUi( this );
+    connect( listBox, SIGNAL( selectionChanged(Q3ListBoxItem*) ), this, SLOT(listBox_selectionChanged(Q3ListBoxItem*) ) );
+}
+
+void GroupConfigGUI::listBox_selectionChanged( Q3ListBoxItem * i)
+{
+    removeBtn->setEnabled(i);
+}
 
 GroupConfigDlg::GroupConfigDlg(QWidget * parent,
       const QString & fileShareGroup, bool restricted,
@@ -169,7 +179,7 @@ void GroupConfigDlg::slotAddUser() {
 void removeList(QList<KUser> & from, const QList<KUser> & that) {
   QList<KUser>::ConstIterator it;
   for ( it = that.begin(); it != that.end(); ++it ) {
-    from.remove(*it);
+    from.removeAll(*it);
   }
 
 }
@@ -188,7 +198,7 @@ bool GroupConfigDlg::addUser(const KUser & user, const KUserGroup & group) {
 
 bool GroupConfigDlg::removeUser(const KUser & user, const KUserGroup & group) {
   QList<KUserGroup> groups = user.groups();
-  groups.remove(group);
+  groups.removeAll(group);
   if (!userMod(user.loginName(),groups)) {
     KMessageBox::sorry(this,i18n("Could not remove user '%1' from group '%2'",
              user.loginName(), group.name()));
@@ -215,11 +225,11 @@ void GroupConfigDlg::slotOk() {
   removeList(removedUsers,m_users);
 
   QList<KUser>::ConstIterator it;
-  for ( it = addedUsers.begin(); it != addedUsers.end(); ++it ) {
+  for ( it = addedUsers.constBegin(); it != addedUsers.constEnd(); ++it ) {
     addUser(*it, m_fileShareGroup);
   }
 
-  for ( it = removedUsers.begin(); it != removedUsers.end(); ++it ) {
+  for ( it = removedUsers.constBegin(); it != removedUsers.constEnd(); ++it ) {
     removeUser(*it, m_fileShareGroup);
   }
 
@@ -240,7 +250,7 @@ void GroupConfigDlg::slotRemoveUser() {
 
   QString loginName = fromPrettyString(item->text());
   KUser user(loginName);
-  m_users.remove(KUser(loginName));
+  m_users.removeAll(KUser(loginName));
   updateListBox();
   m_gui->removeBtn->setEnabled(false);
 }
@@ -361,7 +371,7 @@ void GroupConfigDlg::setFileShareGroup(const KUserGroup & group) {
 bool GroupConfigDlg::addUsersToGroup(QList<KUser> users,const KUserGroup & group) {
   QList<KUser>::ConstIterator it;
   bool result = true;
-  for ( it = users.begin(); it != users.end(); ++it ) {
+  for ( it = users.constBegin(); it != users.constEnd(); ++it ) {
     if (!addUser(*it, group))
       result  = false;
   }
@@ -378,7 +388,7 @@ bool GroupConfigDlg::emptyGroup(const QString & s) {
   bool result = true;
   KUserGroup group(s);
   QList<KUser>::ConstIterator it;
-  for ( it = allUsers.begin(); it != allUsers.end(); ++it ) {
+  for ( it = allUsers.constBegin(); it != allUsers.constEnd(); ++it ) {
     if (!removeUser(*it, group))
       result = false;
   }
