@@ -273,10 +273,17 @@ void DataSourceFactory::start()
         m_startTried = true;
         return;
     }
-    if (!m_size)
-    {
+
+    if (checkLocalFile()) {
+        if (!m_size) {
+            m_startTried = true;
+            findFileSize();
+            return;
+        }
+    } else {
+        //could not create file, maybe device not mounted so abort
         m_startTried = true;
-        findFileSize();
+        changeStatus(Job::Aborted);
         return;
     }
 
@@ -978,6 +985,7 @@ void DataSourceFactory::load(const QDomElement *element)
         m_maxMirrorsUsed = e.attribute("maxMirrorsUsed").toInt(&worked);
         m_maxMirrorsUsed = (worked ? m_maxMirrorsUsed : 3);
     }
+    m_sizeInitiallyDefined = QVariant(e.attribute("sizeInitiallyDefined", "false")).toBool();
 
     //load the finishedChunks
     const QDomElement chunks = e.firstChildElement("chunks");
@@ -999,13 +1007,6 @@ void DataSourceFactory::load(const QDomElement *element)
         {
             m_finishedChunks = new BitSet(data.data(), numBits);
             kDebug(5001) << m_finishedChunks->numOnBits() << " bits on of " << numBits << " bits.";
-            //adapt the downloadedSize to what not has to be downloaded
-            //that might differ to what has been download, e.g. if a chunk did not finish before closing Kget
-            if (m_segSize)
-            {
-                m_downloadedSize = m_segSize * m_finishedChunks->numOnBits();
-                kDebug(5001) << "Modified download size: " << m_downloadedSize;
-            }
         }
 
         //set the finished chunks to started
@@ -1048,16 +1049,14 @@ void DataSourceFactory::changeStatus(Job::Status status, bool loaded)
     m_status = status;
     switch (m_status)
     {
+        case Job::Aborted:
+        case Job::Moving:
         case Job::Stopped:
             m_speed = 0;
             emit speed(m_speed);
             break;
         case Job::Running:
             break;
-        case Job::Moving:
-            m_speed = 0;
-            emit speed(m_speed);
-            break;
         case Job::Finished:
             m_speed = 0;
 
@@ -1085,7 +1084,7 @@ void DataSourceFactory::changeStatus(Job::Status status, bool loaded)
             }
             break;
         default:
-            //TODO handle Delayed and Aborted
+            //TODO handle Delayed
             break;
     }
 
@@ -1139,6 +1138,7 @@ void DataSourceFactory::save(const QDomElement &element)
     factory.setAttribute("status", m_status);
     factory.setAttribute("doDownload", m_doDownload);
     factory.setAttribute("maxMirrorsUsed", m_maxMirrorsUsed);
+    factory.setAttribute("sizeInitiallyDefined", m_sizeInitiallyDefined);
 
     verifier()->save(factory);
     signature()->save(factory);
@@ -1146,8 +1146,13 @@ void DataSourceFactory::save(const QDomElement &element)
     //set the finished chunks
     if (m_finishedChunks)
     {
+        const KIO::fileoffset_t rest = m_size % m_segSize;
+        //the lastSegsize is rest, but only if there is a rest and it is the last segment of the download
+        const KIO::fileoffset_t lastSegSize = (rest ? rest : m_segSize);
+
         //not m_downloadedSize is stored, but the bytes that do not have to be redownloaded
-        factory.setAttribute("processedSize", m_segSize * m_finishedChunks->numOnBits());
+        const bool lastOn = m_finishedChunks->get(m_finishedChunks->getNumBits() - 1);
+        factory.setAttribute("processedSize", m_segSize * (m_finishedChunks->numOnBits() - lastOn) + lastOn * lastSegSize);
 
         QDomElement chunks = doc.createElement("chunks");
         chunks.setAttribute("numBits", m_finishedChunks->getNumBits());
