@@ -30,7 +30,7 @@ Download::~Download()
 
 void Download::slotData(KIO::Job *job, const QByteArray& data)
 {
-    Q_UNUSED(job);
+    Q_UNUSED(job)
     kDebug(5001);
     /**if (data.size() == 0)
     {
@@ -48,7 +48,7 @@ void Download::slotResult(KJob * job)
         case 0://The download has finished
         {
             kDebug(5001) << "Downloading successfully finished" << m_destUrl.url();
-            QFile torrentFile(m_destUrl.path());
+            QFile torrentFile(m_destUrl.toLocalFile());
             if (!torrentFile.open(QIODevice::WriteOnly | QIODevice::Text)) {}
                 //TODO: Do a Message box here
             torrentFile.write(m_data);
@@ -60,7 +60,7 @@ void Download::slotResult(KJob * job)
         case KIO::ERR_FILE_ALREADY_EXIST:
         {
             kDebug(5001) << "ERROR - File already exists";
-            QFile file(m_destUrl.path());
+            QFile file(m_destUrl.toLocalFile());
             emit finishedSuccessfully(m_destUrl, file.readAll());
             m_data = 0;
             break;
