@@ -18,25 +18,28 @@
 #include <Nepomuk/Tag>
 #include <QtCore/QFile>
 
-NepomukHandler::NepomukHandler(Transfer *transfer, QObject *parent)
-  : QObject(parent),
-    m_isValid(false),
+NepomukHandler::NepomukHandler(Transfer *transfer)
+  : QObject(transfer),
     m_transfer(transfer),
     m_destination(transfer->dest()),
     m_resource(Nepomuk::Resource(m_destination, Soprano::Vocabulary::Xesam::File()))
 {
-    isValid();//NOTE the resource is checked whenever changing properties, as the underlying file could be removed etc.
-    //TODO also check isValid() for the const methods? --> set m_resource to mutable?
 }
 
 NepomukHandler::~NepomukHandler()
 {
 }
 
+void NepomukHandler::setNewDestination(const KUrl &newDestination)
+{
+    //TODO look if set tags get copied over!
+    m_destination = newDestination;
+    m_resource = Nepomuk::Resource(m_destination, Soprano::Vocabulary::Xesam::File());
+}
+
 void NepomukHandler::setRating(int rating)
 {
-    if (isValid())
-        m_resource.setRating(rating);
+    m_resource.setRating(rating);
 }
 
 int NepomukHandler::rating() const
@@ -46,28 +49,26 @@ int NepomukHandler::rating() const
 
 void NepomukHandler::addTag(const QString &newTag)
 {
-    if (isValid() && !newTag.isEmpty())
+    if(!newTag.isEmpty())
+    {
         m_resource.addTag(Nepomuk::Tag(newTag));
+    }
 }
 
 void NepomukHandler::addTags(const QStringList &newTags)
 {
-    if (isValid() && !newTags.isEmpty())
+    foreach (const QString &newTag, newTags)
     {
-        QStringList::const_iterator itEnd = newTags.end();
-        for( QStringList::const_iterator it = newTags.begin(); it != itEnd; ++it)
+        if (!newTag.isEmpty())
         {
-            if(!(*it).isEmpty())
-            {
-                m_resource.addTag(Nepomuk::Tag(*it));
-            }
+            m_resource.addTag(Nepomuk::Tag(newTag));
         }
     }
 }
 
 void NepomukHandler::removeTag(const QString &oldTag)
 {
-    if (isValid())
+    if (!oldTag.isEmpty())
     {
         QList<Nepomuk::Tag> list = m_resource.tags();
         list.removeAll(Nepomuk::Tag(oldTag));
@@ -79,14 +80,19 @@ QStringList NepomukHandler::tags() const
 {
     QStringList list;
     foreach (const Nepomuk::Tag &tag, m_resource.tags())
+    {
         list.append(tag.genericLabel());
+    }
     return list;
 }
 
+void NepomukHandler::setProperty(const QUrl &uri, const Nepomuk::Variant &value)
+{
+    m_resource.setProperty(uri, value);
+}
+
 void NepomukHandler::saveFileProperties()
 {
-    if (!isValid())
-        return;
     saveFileProperties(m_resource);
 }
 
@@ -97,26 +103,20 @@ void NepomukHandler::saveFileProperties(const Nepomuk::Resource &res)
     m_res.setProperty(Soprano::Vocabulary::Xesam::size(), Nepomuk::Variant(m_transfer->totalSize()));
 }
 
-bool NepomukHandler::isValid()
+bool NepomukHandler::isValid() const
 {
     bool valid = QFile::exists(m_destination.pathOrUrl());
-//NOTE the assoicated properties to the .part file do not seem to be correctly moved by Nepomuk
-//after finnishing the download by Nepomuk (the metadata is not transferred)
-//TODO reenable when this is fixed
-//     //a part file exists so use that as destination for the Nepomuk properties
-//     if (!valid && (QFile::exists(m_destination.pathOrUrl() + ".part")))
-//     {
-//         valid = true;
-//         m_destination = KUrl::fromPath(m_destination.pathOrUrl() + ".part");
-//         m_resource = Nepomuk::Resource(m_destination, Soprano::Vocabulary::Xesam::File());
-//     }
     valid = valid && m_resource.isValid();
-    //check if the validity changed
-    if (valid != m_isValid)
-    {
-        m_isValid = valid;
-        emit validityChanged(m_isValid);
-    }
 
     return valid;
 }
+
+void NepomukHandler::deinit()
+{
+    if (!isValid())
+    {
+        m_resource.remove();
+    }
+}
+
+#include "nepomukhandler.moc"
