@@ -14,7 +14,7 @@
 
 #include "links.h"
 #include "kget_interface.h"
-#include <config-kgetplugins.h>
+
 #include <KActionCollection>
 #include <KToggleAction>
 #include <kactionmenu.h>
@@ -31,8 +31,8 @@
 #include <dom/html_document.h>
 #include <kparts/partmanager.h>
 
-#ifdef HAVE_WEBKITKDE
-#include <webkitpart.h>
+#ifdef HAVE_KWEBKITPART
+#include <KWebKitPart>
 #include <QWebView>
 #include <QWebFrame>
 #endif
@@ -63,7 +63,7 @@ KGet_plug_in::KGet_plug_in( QObject* parent )
     connect(showSelectedLinksAction, SIGNAL(triggered()), SLOT(slotShowSelectedLinks()));
     menu->addAction(showSelectedLinksAction);
 
-    if (parent && parent->inherits("WebKitPart")) // not available at the moment
+    if (parent && parent->inherits("KWebKitPart")) // not available at the moment
         actionCollection()->action("show_selected_links")->setVisible(false);
 }
 
@@ -79,7 +79,7 @@ void KGet_plug_in::showPopup()
 
     if(QDBusConnection::sessionBus().interface()->isServiceRegistered("org.kde.kget"))
     {
-	OrgKdeKgetInterface kgetInterface("org.kde.kget", "/KGet", QDBusConnection::sessionBus());
+	OrgKdeKgetMainInterface kgetInterface("org.kde.kget", "/KGet", QDBusConnection::sessionBus());
         QDBusReply<bool> reply = kgetInterface.dropTargetVisible();
         if (reply.isValid())
             hasDropTarget = reply.value();
@@ -109,13 +109,13 @@ void KGet_plug_in::slotShowDrop()
         QWidget *parentWidget = 0;
         if (parent() && parent()->inherits("KHTMLPart"))
             parentWidget = static_cast<KHTMLPart*>(parent())->widget();
-#ifdef HAVE_WEBKITKDE
-        if (parent() && parent()->inherits("WebKitPart"))
-            parentWidget = static_cast<WebKitPart*>(parent())->view();
+#ifdef HAVE_KWEBKITPART
+        if (parent() && parent()->inherits("KWebKitPart"))
+            parentWidget = static_cast<KWebKitPart*>(parent())->view();
 #endif
         KRun::runCommand("kget --showDropTarget --hideMainWindow", "kget", "kget", parentWidget);
     } else {
-	OrgKdeKgetInterface kgetInterface("org.kde.kget", "/KGet", QDBusConnection::sessionBus());
+	OrgKdeKgetMainInterface kgetInterface("org.kde.kget", "/KGet", QDBusConnection::sessionBus());
 	kgetInterface.setDropTargetVisible(m_dropTargetAction->isChecked());
     }
 }
@@ -132,10 +132,10 @@ void KGet_plug_in::slotShowSelectedLinks()
 
 void KGet_plug_in::showLinks( bool selectedOnly )
 {
-    if (!parent() || !(parent()->inherits("KHTMLPart") || parent()->inherits("WebKitPart")))
+    if (!parent() || !(parent()->inherits("KHTMLPart") || parent()->inherits("KWebKitPart")))
         return;
 
-    KHTMLPart *htmlPart = dynamic_cast<KHTMLPart*>( parent() );
+    KHTMLPart *htmlPart = qobject_cast<KHTMLPart*>( parent() );
     KParts::Part *activePart = 0L;
     if ( htmlPart && htmlPart->partManager() )
     {
@@ -158,9 +158,9 @@ void KGet_plug_in::showLinks( bool selectedOnly )
         if (htmlPart)
             doc = htmlPart->htmlDocument();
 
-#ifdef HAVE_WEBKITKDE
-        if (parent()->inherits("WebKitPart")) {
-            WebKitPart *part = dynamic_cast<WebKitPart *>(parent());
+#ifdef HAVE_KWEBKITPART
+        KWebKitPart *part = qobject_cast<KWebKitPart *>(parent());
+        if (part) {
             doc.open();
             doc.write(part->view()->page()->currentFrame()->toHtml());
             doc.close();
@@ -223,7 +223,7 @@ void KGet_plug_in::showLinks( bool selectedOnly )
     {
         KToolInvocation::kdeinitExecWait("kget");
     }
-    OrgKdeKgetInterface kgetInterface("org.kde.kget", "/KGet", QDBusConnection::sessionBus());
+    OrgKdeKgetMainInterface kgetInterface("org.kde.kget", "/KGet", QDBusConnection::sessionBus());
     kgetInterface.importLinks(linkList);
 }
 
