@@ -17,7 +17,8 @@
 #include <kstandarddirs.h>
 
 #include "core/kget.h"
-#include "kgetadaptor.h"
+#include "dbus/dbuskgetwrapper.h"
+#include "mainadaptor.h"
 #include "settings.h"
 #include "ui/splash.h"
 #include "mainwindow.h"
@@ -57,9 +58,15 @@ public:
 
         if (!kget)
         {
-            kget = new MainWindow(!args->isSet("showDropTarget"), args->isSet("startWithoutAnimation"));
-            new KgetAdaptor(kget);
-            QDBusConnection::sessionBus().registerObject("/KGet", kget);
+#ifdef DEBUG
+            kget = new MainWindow(!args->isSet("showDropTarget"), args->isSet("startWithoutAnimation"), args->isSet("test"));
+#else
+            kget = new MainWindow(!args->isSet("showDropTarget"), args->isSet("startWithoutAnimation"), false);
+#endif
+
+            DBusKGetWrapper *wrapper = new DBusKGetWrapper(kget);
+            new MainAdaptor(wrapper);
+            QDBusConnection::sessionBus().registerObject("/KGet", wrapper);
         }
         KWindowSystem::activateWindow(kget->winId());
 
@@ -87,19 +94,19 @@ public:
         if (l.count() == 2) {
             KUrl lastUrl = l.last();
             if (lastUrl.isLocalFile()) { // either absolute or relative
-                QString targetPath = lastUrl.path();
-		if (targetPath.startsWith('/')) {
-                    KGet::addTransfer(l.first(), lastUrl.path(), QString());
+                QString targetPath = lastUrl.toLocalFile();
+                if (QDir::isAbsolutePath(targetPath)) {
+                    KGet::addTransfer(l.first(), lastUrl.toLocalFile(), QString());
                  } else {
-		QString fileName = lastUrl.fileName(KUrl::ObeyTrailingSlash);
+                    QString fileName = lastUrl.fileName(KUrl::ObeyTrailingSlash);
                     KGet::addTransfer(l.first(), QString(), fileName);
                 }
-		return 0;
-	    } else if (!lastUrl.isValid() || (lastUrl.scheme().isEmpty() && lastUrl.directory().isEmpty())) {
-	        // Sometimes valid filenames are not recognised by KURL::isLocalFile(), they are marked as invalid then
+                return 0;
+            } else if (!lastUrl.isValid() || (lastUrl.scheme().isEmpty() && lastUrl.directory().isEmpty())) {
+                // Sometimes valid filenames are not recognised by KURL::isLocalFile(), they are marked as invalid then
                 QString suggestedFileName = lastUrl.url();
-		KGet::addTransfer(l.first(), QString(), suggestedFileName);
-		return 0;
+                KGet::addTransfer(l.first(), QString(), suggestedFileName);
+                return 0;
             }
         }
 
@@ -140,11 +147,12 @@ int main(int argc, char *argv[])
                          "(C) 1998 - 2000, Matej Koss"),
                          ki18n("<a href=\"mailto:kget@kde.org\">kget@kde.org</a>"));
 
-    aboutData.addAuthor(ki18n("Dario Massarin"), ki18n("Maintainer, Core Developer"), "nekkar@libero.it");
+    aboutData.addAuthor(ki18n("Lukas Appelhans"), ki18n("Maintainer, Core Developer, Torrent Plugin Author"), "l.appelhans@gmx.de");
+    aboutData.addAuthor(ki18n("Dario Massarin"), ki18n("Core Developer"), "nekkar@libero.it");
     aboutData.addAuthor(ki18n("Urs Wolfer"), ki18n("Core Developer"), "uwolfer@kde.org");
     aboutData.addAuthor(ki18n("Manolo Valdes"), ki18n("Core Developer, Multithreaded Plugin Author"), "nolis71cu@gmail.com");
+    aboutData.addAuthor(ki18n("Matthias Fuchs"), ki18n("Core Developer"), "mat69@gmx.net");
     aboutData.addAuthor(ki18n("Javier Goday"), ki18n("Developer"), "jgoday@gmail.com");
-    aboutData.addAuthor(ki18n("Lukas Appelhans"), ki18n("Developer, Torrent Plugin Author"), "l.appelhans@gmx.de");
     aboutData.addAuthor(ki18n("Patrick Charbonnier"), ki18n("Former Developer"), "pch@freeshell.org");
     aboutData.addAuthor(ki18n("Carsten Pfeiffer"), ki18n("Former Developer"), "pfeiffer@kde.org");
     aboutData.addAuthor(ki18n("Matej Koss"), ki18n("Former Developer"));
@@ -157,6 +165,9 @@ int main(int argc, char *argv[])
     option.add("showDropTarget", ki18n("Start KGet with drop target"));
     option.add("hideMainWindow", ki18n("Start KGet with hidden main window"));
     option.add("startWithoutAnimation", ki18n("Start KGet without drop target animation"));
+#ifdef DEBUG
+    option.add("test", ki18n("Execute Unit Testing"));
+#endif
     option.add("+[URL(s)]", ki18n("URL(s) to download"));
     KCmdLineArgs::addCmdLineOptions(option);
 
