@@ -24,48 +24,46 @@
 #include <util/log.h>
 #include <util/array.h>
 #include <diskio/chunk.h>
+#include <diskio/piecedata.h>
 #include <download/piece.h>
 #include <interfaces/piecedownloader.h>
 
 #include "downloader.h"
 
 namespace bt
-{	
+{
+	DownloadStatus::DownloadStatus() : timeouts(0)
+	{}
 
-	class DownloadStatus : public std::set<Uint32>
+	DownloadStatus::~DownloadStatus()
+	{}
+
+	void DownloadStatus::add(Uint32 p)
 	{
-	public:
-	//	typedef std::set<Uint32>::iterator iterator;
+		status.insert(p);
+	}
 		
-		DownloadStatus()
-		{
+	void DownloadStatus::remove(Uint32 p)
+	{
+		status.remove(p);
+	}
+		
+	bool DownloadStatus::contains(Uint32 p)
+	{
+		return status.contains(p);
+	}
 	
-		}
+	void DownloadStatus::clear()
+	{
+		status.clear();
+	}
 
-		~DownloadStatus()
-		{
-		}
 
-		void add(Uint32 p)
-		{
-			insert(p);
-		}
-		
-		void remove(Uint32 p)
-		{
-			erase(p);
-		}
-		
-		bool contains(Uint32 p)
-		{
-			return count(p) > 0;
-		}
-	};
+	////////////////////////////////////////////////////
 	
 	ChunkDownload::ChunkDownload(Chunk* chunk) : chunk(chunk)
 	{
 		num = num_downloaded = 0;
-		
 		num = chunk->getSize() / MAX_PIECE_LEN;
 		
 		if (chunk->getSize() % MAX_PIECE_LEN != 0)
@@ -80,22 +78,33 @@ namespace bt
 		
 		pieces = BitSet(num);
 		pieces.clear();
+		piece_data = new PieceData* [num]; // array of pointers to the piece data
 		
 		for (Uint32 i = 0;i < num;i++)
-			piece_queue.append(i);
+		{
+			piece_data[i] = 0;
+		}
 		
 		dstatus.setAutoDelete(true);
-		chunk->ref();
 
 		num_pieces_in_hash = 0;
-		if (usingContinuousHashing())
-			hash_gen.start();
-
+		hash_gen.start();
 	}
 
 	ChunkDownload::~ChunkDownload()
 	{
-		chunk->unref();
+		// make sure we do not keep pieces into memory unnecesary
+		for (Uint32 i = 0;i < num;i++)
+		{
+			PieceData* piece = piece_data[i];
+			if (piece)
+			{
+				piece->unref();
+				chunk->savePiece(piece);
+				piece_data[i] = 0;
+			}
+		}
+		delete [] piece_data;
 	}
 
 	bool ChunkDownload::piece(const Piece & p,bool & ok)
@@ -106,18 +115,20 @@ namespace bt
 		Uint32 pp = p.getOffset() / MAX_PIECE_LEN;
 		if (pieces.get(pp))
 			return false;
+
 	
 		DownloadStatus* ds = dstatus.find(p.getPieceDownloader());
 		if (ds)
 			ds->remove(pp);
 		
-		Uint8* buf = chunk->getData();
+		PieceData* buf = chunk->getPiece(p.getOffset(),p.getLength(),false);
 		if (buf)
 		{
+			buf->ref(); 
+			piece_data[pp] = buf;
 			ok = true;
-			memcpy(buf + p.getOffset(),p.getData(),p.getLength());	
+			memcpy(buf->data(),p.getData(),p.getLength());	
 			pieces.set(pp,true);
-			piece_queue.removeAll(pp);
 			piece_providers.insert(p.getPieceDownloader());
 			num_downloaded++;
 			if (pdown.count() > 1)
@@ -125,23 +136,18 @@ namespace bt
 				endgameCancel(p);
 			}
 			
-			if (usingContinuousHashing())
-				updateHash();
+			updateHash();
 			
 			if (num_downloaded >= num)
 			{
 				// finalize hash
-				if (usingContinuousHashing())
-					hash_gen.end();
-
+				hash_gen.end();
 				releaseAllPDs();
 				return true;
 			}
 		}
 		
-		foreach (PieceDownloader* pd,pdown)
-			sendRequests(pd);
-
+		sendRequests();
 		return false;
 	}
 	
@@ -149,8 +155,8 @@ namespace bt
 	{
 		foreach (PieceDownloader* pd,pdown)
 		{
-			sendCancels(pd);
 			pd->release();
+			sendCancels(pd);
 			disconnect(pd,SIGNAL(timedout(const bt::Request& )),this,SLOT(onTimeout(const bt::Request& )));
 			disconnect(pd,SIGNAL(rejected( const bt::Request& )),this,SLOT(onRejected( const bt::Request& )));
 		}
@@ -166,9 +172,9 @@ namespace bt
 		pd->grab();
 		pdown.append(pd);
 		dstatus.insert(pd,new DownloadStatus());
-		sendRequests(pd);
 		connect(pd,SIGNAL(timedout(const bt::Request& )),this,SLOT(onTimeout(const bt::Request& )));
 		connect(pd,SIGNAL(rejected( const bt::Request& )),this,SLOT(onRejected( const bt::Request& )));
+		sendRequests();
 		return true;
 	}
 	
@@ -181,11 +187,30 @@ namespace bt
 			//	Out(SYS_DIO|LOG_DEBUG) << "ds != 0"  << endl;
 			Uint32 p  = r.getOffset() / MAX_PIECE_LEN;
 			ds->remove(p);
+			
+			PieceDownloader* pd = r.getPieceDownloader();
+			if (reject)
+			{
+				// reject, so release the PieceDownloader
+				pd->release();
+				sendCancels(pd);
+				killed(pd);
+			}
+			else
+			{
+				pd->cancel(r); // cancel request
+				ds->timeout();
+				// if we have more then one PieceDownloader and there are timeouts, release it
+				if (ds->numTimeouts() > 0 && pdown.count() > 0)
+				{
+					pd->release();
+					sendCancels(pd);
+					killed(pd);
+				}
+			}
 		}
 			
-			// go over all PD's and do requets again
-		foreach (PieceDownloader* pd,pdown)
-			sendRequests(pd);
+		sendRequests();
 	}
 	
 	void ChunkDownload::onRejected(const Request & r)
@@ -207,50 +232,87 @@ namespace bt
 		}
 	}
 	
-	void ChunkDownload::sendRequests(PieceDownloader* pd)
+	Uint32 ChunkDownload::bestPiece(PieceDownloader* pd)
 	{
-		timer.update();
-		DownloadStatus* ds = dstatus.find(pd);
-		if (!ds)
-			return;
-			
-		// if the peer is choked and we are not downloading an allowed fast chunk
-		if (pd->isChoked())
-			return;
-			
-		Uint32 num_visited = 0;
-		while (num_visited < piece_queue.count() && pd->canAddRequest())
+		Uint32 best = num;
+		Uint32 best_count = 0;
+		// select the piece which is being downloaded the least
+		for (Uint32 i = 0;i < num;i++)
 		{
-			// get the first one in the queue
-			Uint32 i = piece_queue.first();
-			if (!ds->contains(i))
+			if (pieces.get(i))
+				continue;
+			
+			DownloadStatus* ds = dstatus.find(pd);
+			if (ds && ds->contains(i))
+				continue;
+			
+			Uint32 times_downloading = 0;
+			PtrMap<PieceDownloader*,DownloadStatus>::iterator j = dstatus.begin();
+			while (j != dstatus.end())
 			{
-				// send request
-				pd->download(
-						Request(
-							chunk->getIndex(),
-							i*MAX_PIECE_LEN,
-							i+1<num ? MAX_PIECE_LEN : last_size,
-							pd));
-				ds->add(i);
+				if (j->first != pd && j->second->contains(i))
+					times_downloading++;
+				j++;
+			}
+			
+			// nobody is downloading this piece, so return it
+			if (times_downloading == 0)
+				return i;
+			
+			// check if the piece is better then the current best
+			if (best == num || best_count > times_downloading)
+			{
+				best_count = times_downloading;
+				best = i;
 			}
-			// move to the back so that it will take a while before it's turn is up
-			piece_queue.pop_front();
-			piece_queue.append(i);
-			num_visited++;
 		}
 		
-		if (piece_queue.count() < 2 && piece_queue.count() > 0)
-			pd->setNearlyDone(true);
+		return best;
 	}
 	
+	void ChunkDownload::sendRequests()
+	{
+		timer.update();
+		QList<PieceDownloader*> tmp = pdown;
+		
+		while (tmp.count() > 0)
+		{
+			for (QList<PieceDownloader*>::iterator i = tmp.begin();i != tmp.end();)
+			{
+				PieceDownloader* pd = *i;
+				if (!pd->isChoked() && pd->canAddRequest() && sendRequest(pd))
+					i++;
+				else
+					i = tmp.erase(i);
+			}
+		}
+	}
 	
+	bool ChunkDownload::sendRequest(PieceDownloader* pd)
+	{
+		DownloadStatus* ds = dstatus.find(pd);
+		if (!ds || pd->isChoked())
+			return false;
+		
+		// get the best piece to download
+		Uint32 bp = bestPiece(pd);
+		if (bp >= num)
+			return false;
+		
+		pd->download(Request(chunk->getIndex(),bp*MAX_PIECE_LEN,bp+1<num ? MAX_PIECE_LEN : last_size,pd));
+		ds->add(bp);
+		
+		Uint32 left = num - num_downloaded;
+		if (left < 2 && left > 0)
+			pd->setNearlyDone(true);
+		
+		return true;
+	}	
 	
 	void ChunkDownload::update()
 	{
 		// go over all PD's and do requets again
-		foreach (PieceDownloader* pd,pdown)
-			sendRequests(pd);
+		sendRequests();
 	}
 	
 	
@@ -320,7 +382,7 @@ namespace bt
 		}
 		else
 		{
-			return i18n("%1 peers",pdown.count());
+			return i18np("1 peer","%1 peers",pdown.count());
 		}
 	}
 
@@ -340,48 +402,91 @@ namespace bt
 		ChunkDownloadHeader hdr;
 		hdr.index = chunk->getIndex();
 		hdr.num_bits = pieces.getNumBits();
-		hdr.buffered = chunk->getStatus() == Chunk::BUFFERED ? 1 : 0;
+		hdr.buffered = true; // unused 
 		// save the chunk header
 		file.write(&hdr,sizeof(ChunkDownloadHeader));
 		// save the bitset
 		file.write(pieces.getData(),pieces.getNumBytes());
-		if (hdr.buffered)
+		
+		// save how many PieceHeader structs are to be written
+		Uint32 num_pieces_to_follow = 0;
+		for (Uint32 i = 0;i < hdr.num_bits;i++)
+			if (piece_data[i])
+				num_pieces_to_follow++;
+		
+		file.write(&num_pieces_to_follow,sizeof(Uint32));
+			
+		// save all buffered pieces
+		for (Uint32 i = 0;i < hdr.num_bits;i++)
 		{
-			// if it's a buffered chunk, save the contents to
-			file.write(chunk->getData(),chunk->getSize());
-			chunk->clear();
-			chunk->setStatus(Chunk::ON_DISK);
+			if (!piece_data[i])
+				continue;
+			
+			PieceData* pd = piece_data[i];
+			PieceHeader phdr;
+			phdr.piece = i;
+			phdr.size = pd->length();
+			phdr.mapped = pd->mapped() ? 1 : 0;
+			file.write(&phdr,sizeof(PieceHeader));
+			if (!pd->mapped()) // buffered pieces need to be saved
+			{
+				file.write(pd->data(),pd->length());
+			}
 		}
 	}
 		
-	bool ChunkDownload::load(File & file,ChunkDownloadHeader & hdr)
+	bool ChunkDownload::load(File & file,ChunkDownloadHeader & hdr,bool update_hash)
 	{
 		// read pieces
 		if (hdr.num_bits != num)
 			return false; 
 		
 		pieces = BitSet(hdr.num_bits);
-		Array<Uint8> data(pieces.getNumBytes());
-		file.read(data,pieces.getNumBytes());
-		pieces = BitSet(data,hdr.num_bits);
+		file.read(pieces.getData(),pieces.getNumBytes());
+		pieces.updateNumOnBits();
+		
 		num_downloaded = pieces.numOnBits();
-		if (hdr.buffered)
+		Uint32 num_pieces_to_follow = 0;
+		if (file.read(&num_pieces_to_follow,sizeof(Uint32)) != sizeof(Uint32) || num_pieces_to_follow > num)
+			return false;
+		
+		for (Uint32 i = 0;i < num_pieces_to_follow;i++)
 		{
-			// if it's a buffered chunk, load the data to
-			if (file.read(chunk->getData(),chunk->getSize()) != chunk->getSize())
+			PieceHeader phdr;
+			if (file.read(&phdr,sizeof(PieceHeader)) != sizeof(PieceHeader))
 				return false;
+			
+			if (phdr.piece >= num)
+				return false;
+			
+			PieceData* p = chunk->getPiece(phdr.piece * MAX_PIECE_LEN,phdr.size,false);
+			if (!p)
+				return false;
+			
+			p->ref();
+			if (!phdr.mapped)
+			{
+				if (file.read(p->data(),p->length()) != p->length())
+				{
+					p->unref();
+					return false;
+				}
+			}
+			piece_data[phdr.piece] = p;
 		}
 		
-		for (Uint32 i = 0;i < pieces.getNumBits();i++)
-			if (pieces.get(i))
-				piece_queue.removeAll(i);
+		// initialize hash
+		if (update_hash)
+		{
+			num_pieces_in_hash = 0;
+			updateHash();
+		}
 		
-		updateHash();
 		// add a 0 downloader, so that pieces downloaded
 		// in a previous session cannot get a peer banned in this session
 		if (num_downloaded) 
 			piece_providers.insert(0);
-		
+
 		return true;
 	}
 
@@ -448,21 +553,31 @@ namespace bt
 	{
 		// update the hash until where we can
 		Uint32 nn = num_pieces_in_hash;
-		while (pieces.get(nn) && nn < num)
+		while (nn < num && pieces.get(nn))
 			nn++;
 		
 		for (Uint32 i = num_pieces_in_hash;i < nn;i++)
 		{
-			const Uint8* data = chunk->getData() + i * MAX_PIECE_LEN;
-			hash_gen.update(data,i == num - 1 ? last_size : MAX_PIECE_LEN);
+			PieceData* piece = piece_data[i];
+			Uint32 len = i == num - 1 ? last_size : MAX_PIECE_LEN;
+			if (!piece)
+			{
+				piece = chunk->getPiece(i*MAX_PIECE_LEN,len,true);
+				if (piece)
+					piece->ref();
+			}
+			
+			if (piece)
+			{
+				hash_gen.update(piece->data(),len);
+				// save the piece and set it to 0, we no longer need it
+				piece->unref();
+				chunk->savePiece(piece);
+			}
+			piece_data[i] = 0;
 		}
 		num_pieces_in_hash = nn;
 	}
 	
-	bool ChunkDownload::usingContinuousHashing() const
-	{
-		// if the pieces are larger then 1 MB we will be using the continuous hashing feature
-		return pieces.getNumBits() > 64;
-	}
 }
 #include "chunkdownload.moc"
