@@ -19,8 +19,8 @@
     *************************************************************************
 */
 
-#ifndef __KOPETE_MESSAGE_H__
-#define __KOPETE_MESSAGE_H__
+#ifndef KOPETEMESSAGE_H
+#define KOPETEMESSAGE_H
 
 #include <QtCore/QSharedData>
 #include <QtCore/QList>
@@ -99,7 +99,8 @@ public:
 	{
 		TypeNormal, ///< A typical message
 		TypeAction, ///< An IRC-style action.
-		TypeFileTransferRequest ///< A incoming file transfer request message
+		TypeFileTransferRequest, ///< A incoming file transfer request message
+		TypeVoiceClipRequest ///< A incoming voice clip message
 	};
 
 	/**
@@ -257,6 +258,12 @@ public:
 	 */
 	bool isRightToLeft() const;
 
+	/**
+	* returns QStringList with regexp patterns
+	* will be used to look for links in the message
+	*/
+	const QStringList regexpPatterns();
+
 	/**
 	 * @brief Accessor method for the font of the message
 	 * @return The message's font
