@@ -1,16 +1,16 @@
-/*  *************************************************************************
-    *   copyright: (C) 2005 Justin Huff <jjhuff@mspin.net>                  *
-    *************************************************************************
-*/
+/*
+  gsmlib.h  -  SMS Plugin
 
-/*  *************************************************************************
-    *                                                                       *
-    * This program is free software; you can redistribute it and/or modify  *
-    * it under the terms of the GNU General Public License as published by  *
-    * the Free Software Foundation; either version 2 of the License, or     *
-    * (at your option) any later version.                                   *
-    *                                                                       *
-    *************************************************************************
+  Copyright (c) 2005      by Justin Huff        <jjhuff@mspin.net>
+
+  *************************************************************************
+  *                                                                       *
+  * This program is free software; you can redistribute it and/or modify  *
+  * it under the terms of the GNU General Public License as published by  *
+  * the Free Software Foundation; either version 2 of the License, or     *
+  * (at your option) any later version.                                   *
+  *                                                                       *
+  *************************************************************************
 */
 
 #ifndef GSMLIB_H_039562406
