@@ -88,7 +88,7 @@ YahooAccount::YahooAccount(YahooProtocol *parent, const QString& accountId)
 	m_openInboxAction = new KAction( KIcon("mail-folder-inbox"), i18n( "Open Inbo&x..." ), this );
         //, "m_openInboxAction" );
 	QObject::connect(m_openInboxAction, SIGNAL( triggered(bool) ), this, SLOT( slotOpenInbox() ) );
-	m_openYABAction = new KAction( KIcon("x-office-address-book"), i18n( "Open &Addressbook..." ), this );
+	m_openYABAction = new KAction( KIcon("x-office-address-book"), i18n( "Open &Address book..." ), this );
         //, "m_openYABAction" );
 	QObject::connect(m_openYABAction, SIGNAL( triggered(bool) ), this, SLOT( slotOpenYAB() ) );
 	m_editOwnYABEntry = new KAction( KIcon("document-properties"), i18n( "&Edit my contact details..."), this );
@@ -244,6 +244,15 @@ void YahooAccount::initConnectionSignals( enum SignalConnectionType sct )
 		QObject::connect(m_session, SIGNAL(gotBuddy(const QString &, const QString &, const QString &)),
 		                 this, SLOT(slotGotBuddy(const QString &, const QString &, const QString &)));
 
+		QObject::connect(m_session, SIGNAL(buddyAddResult(const QString &, const QString &, bool)),
+				 this, SLOT(slotBuddyAddResult(const QString &, const QString &, bool)));
+
+		QObject::connect(m_session, SIGNAL(buddyRemoveResult(const QString &, const QString &, bool)),
+				 this, SLOT(slotBuddyRemoveResult(const QString &, const QString &, bool)));
+
+		QObject::connect(m_session, SIGNAL(buddyChangeGroupResult(const QString &, const QString &, bool)),
+				 this, SLOT(slotBuddyChangeGroupResult(const QString &, const QString &, bool)));
+
 		QObject::connect(m_session, SIGNAL(authorizationAccepted( const QString & )),
 		                 this, SLOT(slotAuthorizationAccepted( const QString & )) );
 
@@ -335,7 +344,7 @@ void YahooAccount::initConnectionSignals( enum SignalConnectionType sct )
 
 		QObject::connect(m_session, SIGNAL(pictureStatusNotify( const QString&, int )), SLOT(slotPictureStatusNotify( const QString&, int)));
 
-		QObject::connect(m_session, SIGNAL(pictureDownloaded(const QString&, KTemporaryFile*, int)), this, SLOT(slotGotBuddyIcon(const QString&, KTemporaryFile*, int)) );
+		QObject::connect(m_session, SIGNAL(pictureDownloaded(const QString&, const QByteArray &, int)), this, SLOT(slotGotBuddyIcon(const QString&, const QByteArray &, int)) );
 
 		QObject::connect(m_session, SIGNAL(pictureInfoNotify(const QString&, KUrl, int)), this, SLOT(slotGotBuddyIconInfo(const QString&, KUrl, int )));
 
@@ -377,6 +386,15 @@ void YahooAccount::initConnectionSignals( enum SignalConnectionType sct )
 		QObject::disconnect(m_session, SIGNAL(gotBuddy(const QString &, const QString &, const QString &)),
 		                    this, SLOT(slotGotBuddy(const QString &, const QString &, const QString &)));
 
+		QObject::disconnect(m_session, SIGNAL(buddyAddResult(const QString &, const QString &, bool)),
+		                    this, SLOT(slotBuddyAddResult(const QString &, const QString &, bool)));
+
+		QObject::disconnect(m_session, SIGNAL(buddyRemoveResult(const QString &, const QString &, bool)),
+		                    this, SLOT(slotBuddyRemoveResult(const QString &, const QString &, bool)));
+
+		QObject::disconnect(m_session, SIGNAL(buddyChangeGroupResult(const QString &, const QString &, bool)),
+				 this, SLOT(slotBuddyChangeGroupResult(const QString &, const QString &, bool)));
+
 		QObject::disconnect(m_session, SIGNAL(authorizationAccepted( const QString &)),
 		                 this, SLOT(slotAuthorizationAccepted( const QString &)) );
 
@@ -470,7 +488,7 @@ void YahooAccount::initConnectionSignals( enum SignalConnectionType sct )
 
 		QObject::disconnect(m_session, SIGNAL(webcamViewerRequest(const QString&)), this, SLOT(slotWebcamViewerRequest( const QString&)));
 
-		QObject::disconnect(m_session, SIGNAL(pictureDownloaded(const QString&, KTemporaryFile*, int )), this, SLOT(slotGotBuddyIcon(const QString&, KTemporaryFile*,int )));
+		QObject::disconnect(m_session, SIGNAL(pictureDownloaded(const QString&, const QByteArray &, int )), this, SLOT(slotGotBuddyIcon(const QString&, const QByteArray &,int )));
 
 		QObject::disconnect(m_session, SIGNAL(pictureInfoNotify(const QString&, KUrl, int)), this, SLOT(slotGotBuddyIconInfo(const QString&, KUrl, int )));
 
@@ -521,7 +539,7 @@ void YahooAccount::connectWithPassword( const QString &passwd )
 		return;
 	}
 
-	QString server = configGroup()->readEntry( "Server", "scs.msg.yahoo.com" );
+	QString server = configGroup()->readEntry( "Server", "scsa.msg.yahoo.com" );
 	int port = configGroup()->readEntry( "Port", 5050 );
 
 	initConnectionSignals( MakeConnections );
@@ -543,7 +561,7 @@ void YahooAccount::disconnect()
 	if ( isConnected() )
 	{
 		kDebug(YAHOO_GEN_DEBUG) <<  "Attempting to disconnect from Yahoo server ";
-		
+
 		disconnected( Manual );
 		m_session->close();
 		static_cast<YahooContact *>( myself() )->setOnlineStatus( m_protocol->Offline );
@@ -552,6 +570,7 @@ void YahooAccount::disconnect()
 		for ( it = contacts().constBegin(); it != itEnd; ++it )
 			static_cast<YahooContact *>( it.value() )->setOnlineStatus( m_protocol->Offline );
 
+		static_cast<YahooContact*>( myself() )->setOnlineStatus( m_protocol->Offline );
 	}
 	else
 	{       //make sure we set everybody else offline explicitly, just for cleanup
@@ -561,6 +580,8 @@ void YahooAccount::disconnect()
 		QHash<QString,Kopete::Contact*>::ConstIterator it, itEnd = contacts().constEnd();
 		for ( it = contacts().constBegin(); it != itEnd; ++it )
 			static_cast<YahooContact*>( it.value() )->setOnlineStatus( m_protocol->Offline );
+
+		static_cast<YahooContact*>( myself() )->setOnlineStatus( m_protocol->Offline );
 	}
 
 	initConnectionSignals( DeleteConnections );
@@ -622,7 +643,7 @@ void YahooAccount::fillActionMenu( KActionMenu *actionMenu )
 
 YahooContact *YahooAccount::contact( const QString &id )
 {
-	return static_cast<YahooContact *>(contacts()[id]);
+	return static_cast<YahooContact *>(contacts().value( id ));
 }
 
 bool YahooAccount::createContact(const QString &contactId, Kopete::MetaContact *parentContact )
@@ -654,7 +675,7 @@ bool YahooAccount::createChatContact(const QString &nick)
 
 void YahooAccount::sendFile( YahooContact *to, const KUrl &url )
 {
-	QFile file( url.path() );
+	QFile file( url.toLocalFile() );
 
 	Kopete::Transfer *transfer = Kopete::TransferManager::transferManager()->addTransfer ( to,
 		url.fileName(), file.size(), to->userId(), Kopete::FileTransferInfo::Outgoing );
@@ -696,6 +717,7 @@ void YahooAccount::slotLoginResponse( int succ , const QString &url )
 
 		setBuddyIcon( myself()->property( Kopete::Global::Properties::self()->photo() ).value().toString() );
 		m_session->getYABEntries( m_YABLastMerge, m_YABLastRemoteRevision );
+		IDs.clear();
 		m_lastDisconnectCode = 0;
 		theHaveContactList = true;
 		return;
@@ -804,6 +826,40 @@ void YahooAccount::slotGotBuddy( const QString &userid, const QString &alias, co
 		Kopete::Group *g=Kopete::ContactList::self()->findGroup(group);
 		addContact(userid, alias.isEmpty() ? userid : alias, g, Kopete::Account::ChangeKABC);
 	}
+
+	kDebug(YAHOO_GEN_DEBUG) << IDs;
+}
+
+void YahooAccount::slotBuddyAddResult( const QString &userid, const QString &group, bool success )
+{
+     kDebug(YAHOO_GEN_DEBUG) << success;
+
+     if(success)
+	  IDs[userid] = QPair<QString, QString>(group, QString());
+
+     kDebug(YAHOO_GEN_DEBUG) << IDs;
+}
+
+void YahooAccount::slotBuddyRemoveResult( const QString &userid, const QString &group, bool success )
+{
+     kDebug(YAHOO_GEN_DEBUG);
+
+     // Ignore success here, the only reason this will fail is because the
+     // contact isn't on the server's list, so we shouldn't have them in our
+     // list either.
+     IDs.remove(userid);
+
+     kDebug(YAHOO_GEN_DEBUG) << IDs;
+}
+
+void YahooAccount::slotBuddyChangeGroupResult(const QString &userid, const QString &group, bool success)
+{
+     kDebug(YAHOO_GEN_DEBUG);
+     
+     if(success)
+	  IDs[userid] = QPair<QString, QString>(group, QString());
+
+     kDebug(YAHOO_GEN_DEBUG) << IDs;
 }
 
 void YahooAccount::slotAuthorizationAccepted( const QString &who )
@@ -1154,8 +1210,7 @@ void YahooAccount::prepareConference( const QString &who )
 	QHash<QString,Kopete::Contact*>::ConstIterator it, itEnd = contacts().constEnd();
 	for( it = contacts().constBegin(); it != itEnd; ++it )
 	{
-		if( it.value() != myself() )
-			buddies.push_back( it.value()->contactId() );
+		buddies.push_back( it.value()->contactId() );
 	}
 
 	YahooInviteListImpl *dlg = new YahooInviteListImpl( Kopete::UI::Global::mainWidget() );
@@ -1507,31 +1562,23 @@ void YahooAccount::slotGameNotify( const QString & /* who */, int /* stat */ )
 //	kDebug(YAHOO_GEN_DEBUG) ;
 }
 
-void YahooAccount::slotMailNotify( const QString& from, const QString& /* subject */, int cnt )
+void YahooAccount::slotMailNotify( const QString& from, const QString&  subject , int cnt )
 {
-//	kDebug(YAHOO_GEN_DEBUG) << "Mail count: " << cnt;
-
-	if ( cnt > m_currentMailCount && from.isEmpty() )
+	kDebug(YAHOO_GEN_DEBUG) << "Mail count: " << cnt;
+	
+	if ( cnt > 0 && from.isEmpty() )
 	{
-#ifdef __GNUC__
-#warning Fix KNotification here
-#endif
-#if 0
 		QObject::connect(KNotification::event( QLatin1String("yahoo_mail"), i18np( "You have one unread message in your Yahoo inbox.",
 			"You have %1 unread messages in your Yahoo inbox.", cnt ), QPixmap() , 0 ),
 		                 SIGNAL(activated(unsigned int ) ) , this, SLOT( slotOpenInbox() ) );
-#endif
+
 		m_currentMailCount = cnt;
 	}
-	else if ( cnt > m_currentMailCount )
+	else if ( cnt > 0 )
 	{	kDebug(YAHOO_GEN_DEBUG) << "attempting to trigger event";
-#ifdef __GNUC__
-#warning Fix KNotification here
-#endif
-#if 0
-		QObject::connect(KNotification::event( QLatin1String("yahoo_mail"), i18n( "You have a message from %1 in your Yahoo inbox.", from)
+	
+		QObject::connect(KNotification::event( QLatin1String("yahoo_mail"), i18n( "You have a message from %1 in your Yahoo inbox. <br><br>Subject: %2", from, subject )
 		                                       , QPixmap() , 0 ), SIGNAL(activated(unsigned int ) ) , this, SLOT( slotOpenInbox() ) );
-#endif
 		m_currentMailCount = cnt;
 	}
 }
@@ -1627,7 +1674,7 @@ void YahooAccount::slotGotBuddyIconInfo(const QString &who, KUrl url, int checks
 		m_session->downloadPicture( who, url, checksum );
 }
 
-void YahooAccount::slotGotBuddyIcon( const QString &who, KTemporaryFile *file, int checksum )
+void YahooAccount::slotGotBuddyIcon( const QString &who, const QByteArray &data, int checksum )
 {
 	kDebug(YAHOO_GEN_DEBUG) ;
 	YahooContact *kc = contact( who );
@@ -1635,7 +1682,7 @@ void YahooAccount::slotGotBuddyIcon( const QString &who, KTemporaryFile *file, i
 		kDebug(YAHOO_GEN_DEBUG) << "contact " << who << " doesn't exist.";
 		return;
 	}
-	kc->setDisplayPicture( file, checksum );
+	kc->setDisplayPicture( data, checksum );
 }
 void YahooAccount::slotGotBuddyIconRequest( const QString & who )
 {
@@ -1646,9 +1693,9 @@ void YahooAccount::slotGotBuddyIconRequest( const QString & who )
 
 void YahooAccount::setBuddyIcon( const KUrl &url )
 {
-	kDebug(YAHOO_GEN_DEBUG) << "Url: " << url.path();
-	QString s = url.path();
-	if ( url.path().isEmpty() )
+	kDebug(YAHOO_GEN_DEBUG) << "Url: " << url.toLocalFile();
+	QString s = url.toLocalFile();
+	if ( url.toLocalFile().isEmpty() )
 	{
 		myself()->removeProperty( Kopete::Global::Properties::self()->photo() );
 		myself()->removeProperty( YahooProtocol::protocol()->iconRemoteUrl );
@@ -1659,7 +1706,7 @@ void YahooAccount::setBuddyIcon( const KUrl &url )
 	}
 	else
 	{
-		QImage image( url.path() );
+		QImage image( url.toLocalFile() );
 		QString newlocation( KStandardDirs::locateLocal( "appdata", "yahoopictures/"+ url.fileName().toLower() ) ) ;
 		QFile iconFile( newlocation );
 		QByteArray data;
